//
//  tweaktime.cc
//
//  Adjust system time manually and incrementally.
//
//  Copyright (C) 2019 by Matt Roberts,
//  All rights reserved.
//
//  License: GPL 3.0 (www.gnu.org)
//

#include <sys/time.h>
#include <cstring>
#include <iostream>
#include <cmath>

using namespace std;

//
//  usage()
//
static void usage(const std::string &s) {
	int idx = -1;
	for (std::string::const_iterator i = s.begin(); i != s.end(); ++i) {
		if (*i == '/') {
			idx = 1 + (i - s.begin());
		}
	}
	string t;
	if (idx >= 0) {
		t = s.substr(idx);
	} else {
		t = s;
	}
	cerr << "Usage: " << t << " [+|-]<msec>" << endl;
}

//
//  main(...)
//
int main (int argc, char**argv) {
	// check the argument count
	if (argc != 2) {
		usage(argv[0]);
		return 1;
	}

	// check the format of the offset string
	for (int i = 0; i != strlen(argv[1]); ++i) {
		switch (argv[1][i]) {
			case '+':
			case '-':
				if (i != 0) {
					usage(argv[0]);
					return 2;
				}
				break;
			default:
				if (!isdigit(argv[1][i])) {
					usage(argv[0]);
					return 2;
				}
				break;
		}
	}

	// read the offset, and convert to sec
	const double offset = atoi(argv[1]) / 1000.0; // msec -> sec
	if (offset == 0.0)
		return 4;
	
	// DEBUG:
	//cerr << "Offset by " << offset << " seconds." << endl;

	// get the time, advance/retard it, and set it back
	struct timeval now;
	if (::gettimeofday(&now, 0) < 0) {
		cerr << "gettimeofday() call failed: " << ::strerror(errno) << endl;
		return 3;
	}

	// DEBUG:
	//cerr << "Time was: " << now.tv_sec << ":" << now.tv_usec << endl;

	// convert to double seconds
	double now_real = now.tv_sec + (now.tv_usec / 1000000.0);

	// add the requested offset
	now_real += offset;

	// convert back to timeval
	now.tv_sec = static_cast<time_t>(now_real);
	now.tv_usec = static_cast<suseconds_t>((now_real - static_cast<time_t>(now_real)) * 1000000);

	// DEBUG:
	//cerr << "Time now: " << now.tv_sec << ":" << now.tv_usec << endl;

	// set the clock
	if (::settimeofday(&now, 0) < 0) {
		cerr << "settimeofday() call failed: " << ::strerror(errno) << endl;
		return 3;
	}

	return 0;
}
